using System;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Input;
using System.Diagnostics;

namespace HugeFlow.Controls.Interface
{
    // Courtesy of Pete Blois
    public class MouseWheelEventArgs : EventArgs
    {
        /// <summary>
        ///   Ÿ ȯմϴ.
        /// </summary>
        public double Delta { get; private set; }
        private bool handled = false;

        public void SetLastMouseEventArgs(MouseEventArgs value)
        {
            _lastMouseEventArgs = value;
        }

        private MouseEventArgs _lastMouseEventArgs = null;

        public Point GetPosition(UIElement uiElement)
        {
            Debug.Assert(_lastMouseEventArgs != null, "Should only get called when _lastMouseEventArgs is set to something");
            return _lastMouseEventArgs.GetPosition(uiElement);
        }

        public Point OffsetFromPlugin { get; set; }

        public bool AltKey { get; private set; }
        public int CharacterCode { get; private set; }
        public int ClientX { get; private set; }
        public int ClientY { get; private set; }
        public bool CtrlKey { get; private set; }
        public ScriptObject EventObject { get; private set; }
        public string EventType { get; private set; }
        public int KeyCode { get; private set; }
        public MouseButtons MouseButton { get; private set; }
        public int OffsetX { get; private set; }
        public int OffsetY { get; private set; }
        public int ScreenX { get; private set; }
        public int ScreenY { get; private set; }
        public bool ShiftKey { get; private set; }
        public HtmlObject Source { get; private set; }

        /// <summary>
        /// .
        /// </summary>
        /// <param name="delta">  Ÿ</param>
        /// <param name="htmlArgs"> ̺Ʈ Ͼ   HTML ̺Ʈ</param>
        /// <param name="offsetFromPlugin">Plugin ü  콺 Ŀ ġ</param>
        public MouseWheelEventArgs(double delta, HtmlEventArgs htmlArgs, Point offsetFromPlugin)
        {
            this.Delta = delta;

            this.AltKey = htmlArgs.AltKey;
            this.CharacterCode = htmlArgs.CharacterCode;
            this.ClientX = htmlArgs.ClientX;
            this.ClientY = htmlArgs.ClientY;
            this.CtrlKey = htmlArgs.CtrlKey;
            this.EventObject = htmlArgs.EventObject;
            this.EventType = htmlArgs.EventType;
            this.KeyCode = htmlArgs.KeyCode;
            this.MouseButton = htmlArgs.MouseButton;
            this.OffsetX = htmlArgs.OffsetX;
            this.OffsetY = htmlArgs.OffsetY;
            this.ScreenX = htmlArgs.ScreenX;
            this.ScreenY = htmlArgs.ScreenY;
            this.ShiftKey = htmlArgs.ShiftKey;
            this.Source = htmlArgs.Source;

            this.OffsetFromPlugin = offsetFromPlugin;
        }

        // Use handled to prevent the default browser behavior!
        public bool Handled
        {
            get { return this.handled; }
            set { this.handled = value; }
        }
    }

    public class MouseWheelHelper
    {

        public event EventHandler<MouseWheelEventArgs> Moved;
        private static Worker worker;
        private bool isMouseOver = false;
        private MouseEventArgs _latestMouseEventArgs = null;

        public MouseWheelHelper(FrameworkElement element)
        {
            if (MouseWheelHelper.worker == null)
                MouseWheelHelper.worker = new Worker();

            MouseWheelHelper.worker.Moved += this.HandleMouseWheel;

            element.MouseEnter += this.HandleMouseEnter;
            element.MouseLeave += this.HandleMouseLeave;
            element.MouseMove += this.HandleMouseMove;
        }

        private void HandleMouseWheel(object sender, MouseWheelEventArgs args)
        {
            if (this.isMouseOver)
            {
                args.SetLastMouseEventArgs(_latestMouseEventArgs);
                this.Moved(this, args);
            }
        }

        private void HandleMouseEnter(object sender, MouseEventArgs e)
        {
            _latestMouseEventArgs = e;
            this.isMouseOver = true;
        }

        private void HandleMouseLeave(object sender, MouseEventArgs e)
        {
            _latestMouseEventArgs = e;
            this.isMouseOver = false;
        }

        private void HandleMouseMove(object sender, MouseEventArgs e)
        {
            _latestMouseEventArgs = e;
            this.isMouseOver = true;
        }

        private class Worker
        {

            public event EventHandler<MouseWheelEventArgs> Moved;

            public Worker()
            {
                if (HtmlPage.IsEnabled)
                {
                    HtmlPage.Plugin.AttachEvent("DOMMouseScroll", this.HandleMouseWheel);
                    HtmlPage.Plugin.AttachEvent("onmousewheel", this.HandleMouseWheel);
                }
            }

            private void HandleMouseWheel(object sender, HtmlEventArgs args)
            {
                double delta = 0;
                Point offsetFromPlugin = new Point(0, 0);

                ScriptObject eventObj = args.EventObject;

                if (eventObj.GetProperty("wheelDelta") != null)
                {
                    delta = ((double)eventObj.GetProperty("wheelDelta")) / 120;

                    if (HtmlPage.Window.GetProperty("opera") != null)
                        delta = -delta;

                    offsetFromPlugin.X = args.OffsetX;
                    offsetFromPlugin.Y = args.OffsetY;
                }
                else if (eventObj.GetProperty("detail") != null)
                {
                    delta = -((double)eventObj.GetProperty("detail")) / 3;

                    if (HtmlPage.BrowserInformation.UserAgent.IndexOf("Macintosh") != -1)
                        delta = delta * 3;

                    offsetFromPlugin.X = args.ScreenX;
                    offsetFromPlugin.Y = args.ScreenY;
                }

                if (delta != 0 && this.Moved != null)
                {
                    MouseWheelEventArgs wheelArgs = new MouseWheelEventArgs(delta, args, offsetFromPlugin);
                    this.Moved(this, wheelArgs);

                    if (wheelArgs.Handled)
                        args.PreventDefault();
                }
            }
        }
    }
}
