﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using HugeFlow.Controls.Interface;
using System.Diagnostics;

namespace DeepZoomTest1212
{
    public partial class Page : UserControl
    {
        bool _isChecked = false; //마우스가 클릭 되였는지를 체크
        Point _oldMousePosition; //이전 마우스 포인터값 저장

        public Page()
        {
            InitializeComponent();
            cvImage.MouseLeftButtonDown += new MouseButtonEventHandler(cvImage_MouseLeftButtonDown);
            cvImage.MouseLeftButtonUp += new MouseButtonEventHandler(cvImage_MouseLeftButtonUp);
            cvImage.MouseMove += new MouseEventHandler(cvImage_MouseMove);

            MouseWheelHelper mouseWheelHelper = new MouseWheelHelper(cvRoot);
            mouseWheelHelper.Moved += new EventHandler<MouseWheelEventArgs>(mouseWheelHelper_Moved);

            //Viwer설정
            RectangleGeometry rectGeo = new RectangleGeometry();
            rectGeo.Rect = new Rect(0, 0, 720, 600);
            cvRoot.Clip = rectGeo;
        }


        /// <summary>
        /// 휠의 변화에 따른 처리
        /// </summary>

        void mouseWheelHelper_Moved(object sender, MouseWheelEventArgs e)
        {
            e.Handled = true;

            double furturescale = 1;
            if (e.Delta > 0)
                furturescale = tfScaleCvImage.ScaleX * 1.25; //일정하게 확대 하기 위한값
            else if (e.Delta < 0)
                furturescale = tfScaleCvImage.ScaleX * 0.8; //일정하게 축소 하기 위한값


            CheckZoomLevel(ref furturescale); //줌의 크기를 체크해서 1일경우 더이상 축소를 안하게 함

            //줌에 관한 메소드
            ZoomInOut(e.GetPosition(cvImage), furturescale);

            //줌이1인경우 에는 사진을 원래 있던 자리로 이동
            if (furturescale == 1)
                InitPosition();

        }

        private void InitPosition()
        {
            sbTranslatecvImageX.Value = 0;
            sbTranslatecvImageY.Value = 0;
            sbTranslate.Begin();
        }

        private void CheckZoomLevel(ref double zoomScale)
        {
            if (zoomScale < 1)
                zoomScale = 1;

            return;
        }

        private void ZoomInOut(Point centerPoint, double zoomValue)
        {
            cvImage.ChangeRenderTransformOriginByPoint(centerPoint);

            sbScalecvImageX.Value = zoomValue;
            sbScalecvImageY.Value = zoomValue;
            sbScale.Begin();
        }
     
        /// <summary>
        /// 그림객체위에서의 마우스 이동 에 따른 그림의 이동처리
        /// </summary>
        void cvImage_MouseMove(object sender, MouseEventArgs e)
        {
            //TODO: 객체가 끝부분에 왔을때 팅기는 현상 고치기
            
            
            if (_isChecked == false)
                return;

            //마우스 변화에 따른 객체이동
            Point currentPosition = e.GetPosition(cvRoot); // root에서의 mouse좌표
            // cvRoot 의 좌표계로 cvImage의 (0,0) 의 좌표를 구함-즉 pt에는 cvRoot 와 cvImage의 거리값이 나오게 된다. 
            Point ptTemp = cvImage.TransformToVisual(cvRoot).Transform(new Point(0, 0)); 
            Point gapOfMousePoint = new Point((int)(currentPosition.X - _oldMousePosition.X)
                , (int)(currentPosition.Y - _oldMousePosition.Y));

            if ((int)-ptTemp.X < gapOfMousePoint.X)
                gapOfMousePoint.X = -(int)ptTemp.X;

            if ((int)-ptTemp.Y < gapOfMousePoint.Y)
                gapOfMousePoint.Y = -(int)ptTemp.Y;

            int offset_right = (int)(gdTranslate.ActualWidth * tfScaleCvImage.ScaleX
                - ((int)-ptTemp.X + cvRoot.ActualWidth));
            int offset_down = (int)(gdTranslate.ActualHeight * tfScaleCvImage.ScaleY
                - ((int)-ptTemp.Y + cvRoot.ActualHeight));//오르쪽 아래의 공간 

            if (offset_right < -gapOfMousePoint.X)
            {
                gapOfMousePoint.X = -offset_right;
            }

            if (offset_down < -gapOfMousePoint.Y)
            {
                gapOfMousePoint.Y = -offset_down;
            }
            Point translatePoint = new Point(sbTranslatecvImageX.Value, sbTranslatecvImageY.Value);

            translatePoint.X += gapOfMousePoint.X;
            translatePoint.Y += gapOfMousePoint.Y;

            _oldMousePosition = currentPosition;
            TranslateObject(translatePoint);
        }

        /// <summary>
        /// object의 이동 처리
        /// </summary>
        /// <param name="translateValue">이동할 값</param>
        private void TranslateObject(Point translateValue)
        {

            sbTranslatecvImageX.Value = translateValue.X;
            sbTranslatecvImageY.Value = translateValue.Y;
            sbTranslate.Begin();
        }

        void cvImage_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            ///Release MouseCapture and set out _isChecked
            _isChecked = false;
            cvImage.ReleaseMouseCapture();
        }

        void cvImage_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //Set the mouse state : Press=on , CapturetheMouse()
            if (cvImage.CaptureMouse())
                _isChecked = true;

            _oldMousePosition = e.GetPosition(cvRoot);
        }


    }
}
