﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace DeepZoomTest1212
{
    /// <summary>
    /// 
    /// </summary>
    public static class TransformHelper
    {
        /// <summary>
        /// 패널의 RenderTransformOrigin을 튐없이 바꿔주는 메소드
        /// </summary>
        /// <param name="targetPanel">바꿔줄 패널</param>
        /// <param name="centerPoint">바꿔줄 RenderTransformOrigin 위치의 상대좌표</param>
        public static void ChangeRenderTransformOriginByPoint(this FrameworkElement target, Point centerPoint)
        {
            Point renderTransformOringin = target.RenderTransformOrigin; //cvImage 의 Origin 값을 저장
            ScaleTransform scaleTrans = new ScaleTransform();  
            TranslateTransform translateTrans = new TranslateTransform();
            
            if (!target.HasTransformGroup())   
                MakePanelTransformGroup(target);

            foreach (Transform tf in (target.RenderTransform as TransformGroup).Children)
            {
                if (tf is ScaleTransform)
                    scaleTrans = tf as ScaleTransform;
                else if (tf is TranslateTransform)
                    translateTrans = tf as TranslateTransform;
            }
            
            double dOffsetX = (renderTransformOringin.X * target.Width - centerPoint.X) * (1 - scaleTrans.ScaleX);
            double dOffsetY = (renderTransformOringin.Y * target.Height - centerPoint.Y) * (1 - scaleTrans.ScaleY);

            target.RenderTransformOrigin = new Point(centerPoint.X / target.Width, centerPoint.Y / target.Height);

            translateTrans.X += dOffsetX;
            translateTrans.Y += dOffsetY;
        }

        public static bool HasTransformGroup(this UIElement target)
        {
            TransformGroup trGroup = target.RenderTransform as TransformGroup;

            if (trGroup == null)
                return false;

            return true;
        }

        /// <summary>
        /// 패널의 RenderTransform을 TransformGroup으로 만들어 줍니다.
        /// </summary>
        /// <param name="targetPanel"></param>
        public static void MakePanelTransformGroup(this UIElement target)
        {
            // TODO: 기존에 있던 트랜스폼 값을 새로 생성되는 TransformGroup에 넣어주는 코드 필요
            target.RenderTransform = new TransformGroup();
            TransformGroup trGroup = target.RenderTransform as TransformGroup;
   //       trGroup = target.RenderTransform as TransformGroup;
            trGroup.Children.Add(new ScaleTransform());
            trGroup.Children.Add(new TranslateTransform());
        }
    }
}
